/***********************************************************************

DiamondX plugin
Example XEmbed-aware Mozilla browser plugin by Adobe.

Copyright (c) 2007 Adobe Systems Incorporated

 

Permission is hereby granted, free of charge, to any person obtaining a 
copy of this software and associated documentation files (the "Software"), 
to deal in the Software without restriction, including without limitation 
the rights to use, copy, modify, merge, publish, distribute, sublicense, 
and/or sell copies of the Software, and to permit persons to whom the 
Software is furnished to do so, subject to the following conditions:

 

The above copyright notice and this permission notice shall be included in 
all copies or substantial portions of the Software.

 

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS 
OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
THE AUTHORS OR

COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF 
OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE 
SOFTWARE. *
***********************************************************************/

#include "global.h"
#include "swfmoz_player.h"

NPError SwfmozSetWindow(NPP instance, NPWindow* window) {
    SwfmozPlayer* this;
    SwfdecGtkWidget *player_widget;
    int xembedSupported = 0;

    if (instance == NULL)
        return NPERR_INVALID_INSTANCE_ERROR;

    this = (SwfmozPlayer*) instance->pdata;

    if (this == NULL)
        return NPERR_INVALID_INSTANCE_ERROR;

    /* Mozilla likes to re-run its greatest hits */
    if ((window == this->window) &&
        (window->x == this->x) &&
        (window->y == this->y) &&
        (window->width == this->width) &&
        (window->height == this->height)) {
        printf("  (window re-run; returning)\n");
        return NPERR_NO_ERROR;
    }

    NPN_GetValue(instance, NPNVSupportsXEmbedBool, &xembedSupported);
    if (!xembedSupported)
    {
        printf("DiamondX: XEmbed not supported\n");
        return NPERR_GENERIC_ERROR;
    }

    this->window = window;
    this->x = window->x;
    this->y = window->y;
    this->width = window->width;
    this->height = window->height;

    this->container = gtk_plug_new((GdkNativeWindow)window->window);
    player_widget = swfdec_gtk_widget_new(SWFDEC_PLAYER(this));
    GTK_WIDGET_SET_FLAGS (GTK_WIDGET(player_widget), GTK_CAN_FOCUS);

    /* all the events that our canvas wants to receive */
    gtk_widget_add_events(
        GTK_WIDGET(player_widget),
        GDK_BUTTON_PRESS_MASK | 
        GDK_BUTTON_RELEASE_MASK |
        GDK_KEY_PRESS_MASK | 
        GDK_KEY_RELEASE_MASK | 
        GDK_POINTER_MOTION_MASK |
        GDK_SCROLL_MASK |
        GDK_EXPOSURE_MASK |
        GDK_VISIBILITY_NOTIFY_MASK |
        GDK_ENTER_NOTIFY_MASK |
        GDK_LEAVE_NOTIFY_MASK |
        GDK_FOCUS_CHANGE_MASK
    );

    gtk_widget_show(GTK_WIDGET(player_widget));
    gtk_container_add(GTK_CONTAINER(this->container), GTK_WIDGET(player_widget));
    gtk_widget_show(this->container);

    return NPERR_NO_ERROR;
}
